package javax.swing.text.rtf;

import java.io.IOException;
import java.io.PrintStream;
import java.io.ByteArrayOutputStream;

abstract class RTFParser extends AbstractFilter
{
    public int level;
    private int state;
    private StringBuffer currentCharacters;
    private String pendingKeyword;
    private int pendingCharacter;
    private long binaryBytesLeft;
    ByteArrayOutputStream binaryBuf;
    private boolean[] savedSpecials;
    protected PrintStream warnings;
    private final int S_text = 0;
    private final int S_backslashed = 1;
    private final int S_token = 2;
    private final int S_parameter = 3;
    private final int S_aftertick = 4;
    private final int S_aftertickc = 5;
    private final int S_inblob = 6;
    static final boolean[] rtfSpecialsTable;
    
    public abstract boolean handleKeyword(String p0);
    
    public abstract boolean handleKeyword(String p0, int p1);
    
    public abstract void handleText(String p0);
    
    public void handleText(char c) {
        this.handleText(String.valueOf(c));
    }
    
    public abstract void handleBinaryBlob(byte[] p0);
    
    public abstract void begingroup();
    
    public abstract void endgroup();
    
    public RTFParser() {
        this.currentCharacters = new StringBuffer();
        this.state = 0;
        this.pendingKeyword = null;
        this.level = 0;
        this.specialsTable = RTFParser.rtfSpecialsTable;
    }
    
    public void writeSpecial(int n) throws IOException {
        this.write((char)n);
    }
    
    protected void warning(String x) {
        if (this.warnings != null) {
            this.warnings.println(x);
        }
    }
    
    @Override
    public void write(String substring) throws IOException {
        if (this.state != 0) {
            int n;
            int length;
            for (n = 0, length = substring.length(); n < length && this.state != 0; ++n) {
                this.write(substring.charAt(n));
            }
            if (n >= length) {
                return;
            }
            substring = substring.substring(n);
        }
        if (this.currentCharacters.length() > 0) {
            this.currentCharacters.append(substring);
        }
        else {
            this.handleText(substring);
        }
    }
    
    public void write(char b) throws IOException {
        switch (this.state) {
            case 0: {
                if (b != '\n') {
                    if (b == '\r') {
                        break;
                    }
                    if (b == '{') {
                        if (this.currentCharacters.length() > 0) {
                            this.handleText(this.currentCharacters.toString());
                            this.currentCharacters = new StringBuffer();
                        }
                        ++this.level;
                        this.begingroup();
                        break;
                    }
                    else if (b == '}') {
                        if (this.currentCharacters.length() > 0) {
                            this.handleText(this.currentCharacters.toString());
                            this.currentCharacters = new StringBuffer();
                        }
                        if (this.level == 0) {
                            throw new IOException("Too many close-groups in RTF text");
                        }
                        this.endgroup();
                        --this.level;
                        break;
                    }
                    else if (b == '\\') {
                        if (this.currentCharacters.length() > 0) {
                            this.handleText(this.currentCharacters.toString());
                            this.currentCharacters = new StringBuffer();
                        }
                        this.state = 1;
                        break;
                    }
                    else {
                        this.currentCharacters.append(b);
                        break;
                    }
                }
                else {
                    break;
                }
            }
            case 1: {
                if (b == '\'') {
                    this.state = 4;
                    break;
                }
                if (!Character.isLetter(b)) {
                    char[] array = { b };
                    if (!this.handleKeyword(new String(array))) {
                        this.warning("Unknown keyword: " + array + " (" + (int)(byte)b + ")");
                    }
                    this.state = 0;
                    this.pendingKeyword = null;
                    break;
                }
                else {
                    this.state = 2;
                }
            }
            case 2: {
                if (Character.isLetter(b)) {
                    this.currentCharacters.append(b);
                    break;
                }
                this.pendingKeyword = this.currentCharacters.toString();
                this.currentCharacters = new StringBuffer();
                if (Character.isDigit(b) || b == '-') {
                    this.state = 3;
                    this.currentCharacters.append(b);
                    break;
                }
                if (!this.handleKeyword(this.pendingKeyword)) {
                    this.warning("Unknown keyword: " + this.pendingKeyword);
                }
                this.pendingKeyword = null;
                this.state = 0;
                if (!Character.isWhitespace(b)) {
                    this.write(b);
                    break;
                }
                break;
            }
            case 3: {
                if (Character.isDigit(b)) {
                    this.currentCharacters.append(b);
                    break;
                }
                if (this.pendingKeyword.equals("bin")) {
                    long long1 = Long.parseLong(this.currentCharacters.toString());
                    this.pendingKeyword = null;
                    this.state = 6;
                    this.binaryBytesLeft = long1;
                    if (this.binaryBytesLeft > 2147483647L) {
                        this.binaryBuf = new ByteArrayOutputStream(Integer.MAX_VALUE);
                    }
                    else {
                        this.binaryBuf = new ByteArrayOutputStream((int)this.binaryBytesLeft);
                    }
                    this.savedSpecials = this.specialsTable;
                    this.specialsTable = RTFParser.allSpecialsTable;
                    break;
                }
                else {
                    if (!this.handleKeyword(this.pendingKeyword, Integer.parseInt(this.currentCharacters.toString()))) {
                        this.warning("Unknown keyword: " + this.pendingKeyword + " (param " + this.currentCharacters + ")");
                    }
                    this.pendingKeyword = null;
                    this.currentCharacters = new StringBuffer();
                    this.state = 0;
                    if (!Character.isWhitespace(b)) {
                        this.write(b);
                    }
                    break;
                }
            }
            case 4: {
                if (Character.digit(b, 16) == -1) {
                    this.state = 0;
                    break;
                }
                this.pendingCharacter = Character.digit(b, 16);
                this.state = 5;
                break;
            }
            case 5: {
                this.state = 0;
                if (Character.digit(b, 16) != -1) {
                    this.pendingCharacter = this.pendingCharacter * 16 + Character.digit(b, 16);
                    int n = this.translationTable[this.pendingCharacter];
                    if (n != '\0') {
                        this.handleText((char)n);
                        break;
                    }
                    break;
                }
                else {
                    break;
                }
            }
            case 6: {
                this.binaryBuf.write((int)b);
                --this.binaryBytesLeft;
                if (this.binaryBytesLeft == 0L) {
                    this.state = 0;
                    this.specialsTable = this.savedSpecials;
                    this.savedSpecials = null;
                    this.handleBinaryBlob(this.binaryBuf.toByteArray());
                    this.binaryBuf = null;
                    break;
                }
                break;
            }
        }
    }
    
    @Override
    public void flush() throws IOException {
        super.flush();
        if (this.state == 0 && this.currentCharacters.length() > 0) {
            this.handleText(this.currentCharacters.toString());
            this.currentCharacters = new StringBuffer();
        }
    }
    
    @Override
    public void close() throws IOException {
        this.flush();
        if (this.state != 0 || this.level > 0) {
            this.warning("Truncated RTF file.");
            while (this.level > 0) {
                this.endgroup();
                --this.level;
            }
        }
        super.close();
    }
    
    static {
        (rtfSpecialsTable = (boolean[])RTFParser.noSpecialsTable.clone())[10] = true;
        RTFParser.rtfSpecialsTable[13] = true;
        RTFParser.rtfSpecialsTable[123] = true;
        RTFParser.rtfSpecialsTable[125] = true;
        RTFParser.rtfSpecialsTable[92] = true;
    }
}
